<?php

namespace LakeDrops\Component\Dotenv;

use Composer\IO\IOInterface;
use Symfony\Component\Dotenv\Dotenv as SymfonyDotenv;

/**
 * Manages .env files.
 */
final class Dotenv {

  /**
   * Name of the plugin "owning" this instance, used as a prefix.
   *
   * @var string
   */
  private $name;

  /**
   * The input-output object of the composer session.
   *
   * @var \Composer\IO\IOInterface
   */
  private $io;

  /**
   * The Symfony Dotenv object.
   *
   * @var \Symfony\Component\Dotenv\Dotenv
   */
  private $dotenv;

  /**
   * Dotenv constructor.
   *
   * @param string $name
   *   Name of the plugin "owning" this instance, used as a prefix.
   * @param \Composer\IO\IOInterface $io
   *   The input-output object of the composer session.
   */
  public function __construct($name, IOInterface $io) {
    $this->name = strtoupper($name);
    $this->io = $io;
    $this->dotenv = new SymfonyDotenv();
    $this->load();
  }

  /**
   * Receive a value from the environment.
   *
   * First of all, it prefixes the key with the plugin name and converts the
   * result into upper case. Then looking for the value with PHP's getenv and if
   * nothing found, it prompts the user for input if the session is interactive.
   * Finally, if the value is still not available, the default value is being
   * used.
   *
   * @param string $key
   *   The key of the environment variable.
   * @param string $prompt
   *   The text being displayed when prompting the user.
   * @param string $default
   *   The default value.
   *
   * @return string
   *   The value.
   */
  public function receive($key, $prompt, $default = '') {
    $key = $this->name . '_' . strtoupper($key);
    $value = getenv($key);
    if (empty($value)) {
      if ($this->io->isInteractive()) {
        $value = $this->io->ask($prompt . ': ');
      }
      if (empty($value)) {
        $value = $default;
      }
      $this->put($key, $value);
    }
    return $value;
  }

  /**
   * Set an environment variable.
   *
   * Sets the environment variable into a .env file only if there isn't an
   * environment variable present yet. The .env file will also be added to the
   * .gitignore file.
   *
   * @param string $key
   *   The name of the environment variable.
   * @param string $value
   *   The value of the environment variable.
   */
  public function put($key, $value) {
    $key = strtoupper($key);
    if (!empty(getenv($key))) {
      return;
    }
    $env = file_exists('.env') ? file_get_contents('.env') : '';
    $env .= PHP_EOL . $key . '=' . $value . PHP_EOL;
    file_put_contents('.env', $env);
    $this->load();
    passthru(escapeshellcmd('git -c "user.email=dotenv@lakedrops.com" ignore .env'));
  }

  /**
   * Load the local .env file.
   */
  private function load() {
    try {
      $this->dotenv->load('.env');
    }
    catch (\Exception $ex) {
      // Ignore.
    }
  }

}
